<xsl:stylesheet xmlns:xsl="http://www.w3.org/1999/XSL/Transform"
    xmlns:xs="http://www.w3.org/2001/XMLSchema"
    xmlns="http://dita.oasis-open.org/architecture/2005/"
    xmlns:ditaarch="http://dita.oasis-open.org/architecture/2005/"
    xmlns:ditaaccess="java:ro.sync.ecss.dita.extensions.DITAXSLTExtensionFunctionUtil"
    exclude-result-prefixes="#all" version="3.0">

  <!-- returns the element referenced by an <xref> or <keyref> -->
  <xsl:include href="util-get-referenced-element.xsl"/>

  <!-- identity transformation -->
  <xsl:mode on-no-match="shallow-copy"/>


  <!-- update @type attributes in <xref> and <link> elements -->
  <xsl:template match="(xref|link)[@href or @keyref]">
    <xsl:variable name="result" as="element()">
      <xsl:next-match/>  <!-- apply other templates first -->
    </xsl:variable>
    <xsl:variable name="that-elt" as="element()?">
      <xsl:apply-templates select="." mode="get-referenced-element"/>  <!-- must use original element so the base-uri() context is correct -->
    </xsl:variable>
    <xsl:copy select="$result">
      <xsl:apply-templates select="$result/@*" mode="#current"/>
      <xsl:choose>
        <xsl:when test="not($that-elt)"/>  <!-- reference did not resolve - do nothing -->
        <xsl:when test="$result/@type = local-name($that-elt)"/>  <!-- @type is correct - do nothing -->
        <xsl:when test="empty($result/@type) or $result/@type != local-name($that-elt)">  <!-- @type is missing or incorrect - always fix -->
          <xsl:attribute name="type" select="local-name($that-elt)"/>
        </xsl:when>
      </xsl:choose>
      <xsl:apply-templates select="$result/node()" mode="#current"/>
    </xsl:copy>
  </xsl:template>


  <!-- set cross-book link target text for <xref> or <link> elements that
     * are empty
     * are not empty, but have <?oxy-peertext?> PIs to allow updates -->
  <xsl:template match="(xref | link)
      [contains(@keyref, '.')]
      [ditaaccess:getKeyRefInfo(@keyref, base-uri(.)) = ('PEER_MAP_KEY_HREF')]
      [empty(node()) or processing-instruction('oxy-peertext')]">
    <xsl:variable name="result" as="element()">
      <xsl:next-match/>  <!-- apply other templates first -->
    </xsl:variable>

    <!-- get existing title contents -->
    <xsl:variable name="existing-title-contents" select="$result/node()" as="node()*"/>

    <!-- get title contents from peer map @keyref -->
    <xsl:variable name="new-title-contents">
      <xsl:variable name="target-elt" as="element()?">
        <xsl:apply-templates select="." mode="get-referenced-element"/>  <!-- must use original element so the base-uri() context is correct -->
      </xsl:variable>
      <xsl:apply-templates select="$target-elt/title" mode="get-title-contents"/>
    </xsl:variable>
    <xsl:copy select="$result">
      <xsl:apply-templates select="@*"/>
      <xsl:choose>
        <!-- keep original title contents if
           * new title is empty (@keyref did not resolve)
           * new title has @href/@keyref or profiling condition attributes
           * normalized new title matches normalized existing title -->
        <xsl:when test="
            empty($new-title-contents)
            or $new-title-contents//(@href | @keyref | @audience | @platform | @product | @props)
            or (normalize-space(serialize($new-title-contents)) eq normalize-space(serialize($existing-title-contents)))">
          <xsl:sequence select="$existing-title-contents"/>
        </xsl:when>
        <!-- use new title -->
        <xsl:otherwise>
          <xsl:sequence select="$new-title-contents"/>
        </xsl:otherwise>
      </xsl:choose>
    </xsl:copy>
  </xsl:template>

  <!-- resolve <title> content to suitable target text for inside an <xref> or <link> -->
  <xsl:mode name="get-title-contents" on-no-match="shallow-copy"/>

  <!-- get contents of <title>, plus add a <?oxy-peertext?> PI -->
  <xsl:template match="title" mode="get-title-contents">
    <xsl:apply-templates select="node()" mode="#current"/>
    <xsl:processing-instruction name="oxy-peertext"/>
  </xsl:template>

  <!-- simplify linefeeds, indents to a space
     (like normalize-space(), but we leave leading/trailing whitespace in place) -->
  <xsl:template match="text()" mode="get-title-contents">
    <xsl:value-of select="replace(., '\s{2,}', ' ')"/>
  </xsl:template>

  <!-- copy elements without their namespaces -->
  <xsl:template match="*" mode="get-title-contents">
    <xsl:copy select="." copy-namespaces="no">
      <xsl:apply-templates select="@* | node()" mode="#current"/>
    </xsl:copy>
  </xsl:template>

</xsl:stylesheet>
