<?xml version="1.0" encoding="UTF-8"?>
<schema xmlns="http://purl.oclc.org/dsdl/schematron" queryBinding="xslt3"
  xmlns:sqf="http://www.schematron-quickfix.com/validator/process"
  xmlns:xsl="http://www.w3.org/1999/XSL/Transform">
  <ns uri="java:ro.sync.ecss.dita.extensions.DITAXSLTExtensionFunctionUtil"
    prefix="ditaaccess"/>

  <xsl:include href="../refactoring/util-get-referenced-element.xsl"/>

  <!-- some debug informational checks to verify that Schematron file is active (commented by default) -->
  <!--
  <pattern>
    <rule context="*[@keyref]" role="info">
      <report test="true()">getKeyRefInfo(<value-of select="@keyref"/>) == <value-of select="ditaaccess:getKeyRefInfo(@keyref, base-uri(.))"/></report>
    </rule>
  </pattern>

  <pattern>
    <rule context="*[@keyref]" role="info">
      <report test="true()">getKeyRefAbsoluteReference(<value-of select="@keyref"/>) == <value-of select="ditaaccess:getKeyRefAbsoluteReference(@keyref, base-uri(.))"/></report>
    </rule>
  </pattern>
  -->

  <pattern id="refs-cross-book">
    <rule context="(link|xref)[contains(@keyref, '.')][ditaaccess:getKeyRefInfo(@keyref, base-uri(.)) = 'PEER_MAP_KEY_HREF']" role="error">
      <report test="empty(node())" sqf:fix="fix-target">Empty cross-book reference; please run the "Update cross references" refactoring operation.</report>
      <sqf:fix id="fix-target">
        <sqf:description>
          <sqf:title>Add the cross-book link target text.</sqf:title>
        </sqf:description>
        <sqf:add position="first-child"><xsl:apply-templates mode="get-target-text" select="."/></sqf:add>
      </sqf:fix>
    </rule>
  </pattern>

  <pattern>
    <rule context="(xref|link)
        [contains(@keyref, '.')][ph[@keyref]]" role="error">
      <report test="ditaaccess:getKeyRefInfo(@keyref, base-uri(.)) eq 'PEER_MAP_KEY_HREF'">Variables should not be used in cross-book link text.</report>
    </rule>
  </pattern>


  <!-- obtain target text content for an <xref> or <link> element -->
  <xsl:template match="(xref|link)" mode="get-target-text">

    <!-- get referenced element -->
    <xsl:variable name="target-elt" as="element()?">
      <xsl:apply-templates select="." mode="get-referenced-element"/>
    </xsl:variable>

    <!-- compute target text content from its <title> element -->
    <xsl:apply-templates select="$target-elt/title" mode="get-title-contents"/>
  </xsl:template>

  <!-- resolve <title> content to suitable target text for inside an <xref> or <link> -->
  <xsl:mode name="get-title-contents" on-no-match="shallow-copy"/>

  <!-- get contents of <title>, plus add a <?oxy-peertext?> PI -->
  <xsl:template match="title" mode="get-title-contents">
    <xsl:apply-templates select="node()" mode="#current"/>
    <xsl:processing-instruction name="oxy-peertext"/>
  </xsl:template>

  <!-- simplify linefeeds, indents to a space
       (like normalize-space(), but we leave leading/trailing whitespace in place) -->
  <xsl:template match="text()" mode="get-title-contents">
    <xsl:value-of select="replace(., '\s{2,}', ' ')"/>
  </xsl:template>

  <!-- copy elements without their namespaces -->
  <xsl:template match="*" mode="get-title-contents">
    <xsl:copy select="." copy-namespaces="no">
      <xsl:apply-templates select="@* | node()" mode="#current"/>
    </xsl:copy>
  </xsl:template>

</schema>
